
/*
*===========================================================
*        File Name: adp5065.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "adp5065.h"
#include "hardware.h"


static unsigned char adp5065_rd(unsigned char regid);
static void adp5065_wr(unsigned char regid, unsigned char data);

const ADP5065_SFR_T adp5065_default = {
	//@: change from power on reset value
	.reg = {
		.VINxSet.BITs.ILIM = 0x0A,		//1000mA	@
		.VINxSet.BITs.Reserve = 0x00,	//Res	
		
		.TermSet.BITs.IEND = 0x01,	//52.5mA	
		.TermSet.BITs.VTRM = 0x23,	//4.2V	
		
		.ChargeCur.BITs.ITRK_DEAD = 0x02,	//20mA	
		.ChargeCur.BITs.ICHG = 0x03,	//850 mA 	@
		.ChargeCur.BITs.Reserve = 0x00,			
		.ChargeCur.BITs.C_10EOC = 0x00,	//Disable C/20 EOC	
		.ChargeCur.BITs.C_20EOC = 0x00,	//Disable C/10 EOC	

		.VoltThr.BITs.VWEAK = 0x03,			//3.0 V.	
		.VoltThr.BITs.VTRK_DEAD = 0x00,	//2.4 V.	
		.VoltThr.BITs.VRCH = 0x03,			//260 mV.
		.VoltThr.BITs.Reserve = 0x00,
		
		.TimerSet.BITs.RESET_WD = 0x01,				//High resets the watchdog safety timer
		.TimerSet.BITs.WD_PERIOD = 0x00,			//t_wd = 32sec	
		.TimerSet.BITs.EN_WD = 0x01,					//Enable Watchdog Timer @
		.TimerSet.BITs.CHG_TMR_PERIOD = 0x01,	//Tricle/Charge Timer Period: 64sec/600min
		.TimerSet.BITs.EN_CHG_TIMER = 0x01,		//Enable Tricle/Charge Timer	
		.TimerSet.BITs.EN_TEND = 0x00,				//Disable t_end timer	
		.TimerSet.BITs.Reserve = 0x00,
		
		.FuncSet1.BITs.EN_CHG = 0x00,					//Disable Charge for default status	 @
		.FuncSet1.BITs.EN_TRK = 0x01,					//Enable Trickle Charge	
		.FuncSet1.BITs.EN_EOC = 0x01,					//Enable End of Charge	??
		.FuncSet1.BITs.Reserve = 0x00,
		.FuncSet1.BITs.EN_THR = 0x01,					//Enable thermal monitoring current source @	
		.FuncSet1.BITs.EN_BMON = 0x01,				//Enable Battery Mornitoring @
		.FuncSet1.BITs.DIS_IPK_SD = 0x01,			//Disable Auto Shutdown & Onlu Set Bit IPK_STAT.	
		.FuncSet1.BITs.EN_JEITA = 0x00,				//Disable JEITA 		
		
		.IntEnSet.BITs.EN_VIN_INT = 0x00,			//Disable this Interrupt	
		.IntEnSet.BITs.EN_CHG_INT = 0x00,			//Disable this Interrupt		
		.IntEnSet.BITs.EN_BAT_INT = 0x00,			//Disable this Interrupt		
		.IntEnSet.BITs.EN_THR_INT = 0x00,			//Disable this Interrupt	
		.IntEnSet.BITs.EN_TSD_INT = 0x00,			//Disable this Interrupt		
		.IntEnSet.BITs.EN_WD_INT  = 0x00,			//Disable this Interrupt		
		.IntEnSet.BITs.EN_THERM_LIM_INT = 0x00,	//Disable this Interrupt		
		.IntEnSet.BITs.EN_IND_PEAK_INT  = 0x00,	//Disable this Interrupt		
		
		.FaultReg.BITs.TSD_140  = 0x01,				//Clear this bit when intitial	
		.FaultReg.BITs.TSDL_130 = 0x01,				//Clear this bit when intitial		
		.FaultReg.BITs.IND_PEAK_INT = 0x01,		//Clear this bit when intitial		
		.FaultReg.BITs.BAT_SHR = 0x01,				//Clear this bit when intitial		
		
		.BatShort.BITs.VBAT_SHR = 0x04,				//Battery Short voltage threshold: 2.4V	
		.BatShort.BITs.Reserve = 0x00,
		.BatShort.BITs.TBAT_SHR = 0x00				//Battery Short timerout timer : 1sec @	
	}
};

ADP5065_T adp5065 ={
	.rd = adp5065_rd,
	.wr = adp5065_wr,
	.id_writable = {0x02,0x03,0x04,0x05,0x06,0x07,0x09,0x0D,0x10},
	.id_readable = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x09,0x0A,0x0B,0x0C,0x0D,0x10}
};

static unsigned char adp5065_rd(unsigned char regid){
	return(i2c.x2id_rd(I2CID_ADP5065,regid));
}
static void adp5065_wr(unsigned char regid, unsigned char data){
	i2c.x2id_wr(I2CID_ADP5065,regid,data);
}


// TypeDef Battery


