
/*
*===========================================================
*        File Name: cn0352.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "uart.h"
#include "timer0.h"
#include "timer1.h"
#include "cn0352.h"
#include "i2c.h"
#include "hardware.h"
#include "ctrlport.h"
#include "dac.h"
#include "adc.h"
#include "task.h"
#include "cmd.h"
#include "pwm.h"
#include "common\DacLib.h"
#include "common\ClkLib.h"
#include "common\WdtLib.h"
#include "monitor.h"




//Initialization
static void cn0352_run_cfg_init(void);
static void cn0352_run_cfg_adp5065_init(void);
static void cn0352_run_cfg_baudrateupd(void);
static unsigned char cn0352_run_cfg_moduleIDupd(void);
//cfg
static void cn0352_run_cfg_InCurrLimit(CHANNEL_T *ptr, unsigned char data);
static void cn0352_run_cfg_TrickleVth(CHANNEL_T *ptr, unsigned char data);
static void cn0352_run_cfg_TrickleCurr(CHANNEL_T *ptr, unsigned char data);
static void cn0352_run_cfg_FastCurr(CHANNEL_T *ptr, unsigned char data);
static void cn0352_run_cfg_TermVolt(CHANNEL_T *ptr, unsigned char data);
static void cn0352_run_cfg_TermCurr(CHANNEL_T *ptr, unsigned char data);
static void cn0352_run_cfg_DisTermVolt(CHANNEL_T *ptr, unsigned char data[3]);
static void cn0352_run_cfg_DisTermCurr(CHANNEL_T *ptr, unsigned char data[3]);
static void cn0352_run_cfg_CHGStart(CHANNEL_T *ptr);
static void cn0352_run_cfg_CHGStop(CHANNEL_T *ptr);
static void cn0352_run_cfg_DHGStart(CHANNEL_T *ptr);
static void cn0352_run_cfg_DHGStop(CHANNEL_T *ptr);
static void cn0352_run_cfg_sw_reset(void);

static void cn0352_run_cfg_InCurrLimit_Allset(unsigned char data);
static void cn0352_run_cfg_TrickleVth_Allset(unsigned char data);
static void cn0352_run_cfg_TrickleCurr_Allset(unsigned char data);
static void cn0352_run_cfg_FastCurr_Allset(unsigned char data);
static void cn0352_run_cfg_TermVolt_Allset(unsigned char data);
static void cn0352_run_cfg_TermCurr_Allset(unsigned char data);
static void cn0352_run_cfg_DisTermVolt_Allset(unsigned char data[3]);
static void cn0352_run_cfg_DisTermCurr_Allset(unsigned char data[3]);
static void cn0352_run_cfg_test_cycle(unsigned short data);
static void cn0352_run_cfg_CHG_CFG_ALL(FORMAT_CFG_ALL_T* cfg);
static void cn0352_run_cfg_CHGStart_Allset(void);
static void cn0352_run_cfg_CHGStop_Allset(void);
static void cn0352_run_cft_adp5065_falut_recover_channel(CHANNEL_T *ptr);
static void cn0352_run_cfg_adp5065_falut_recover(void);
//static void cn0352_run_cfg_DHGStart_Allset(void);
//static void cn0352_run_cfg_DHGStop_Allset(void);


//Task Reference
static void cn0352_run_cfg_start_volt_curr(void);
static void cn0352_run_cfg_start_temp_vdac(void);
static void cn0352_run_cfg_start_temp_sync(void);
static void cn0352_task_Vdisc_Refresh(void);
static void cn0352_task_BAT_V_I_RB(void);
static void cn0352_task_BAT_T_VDIS_RB(void);
static void cn0352_task_BAT_T_SYNC_RB(void);
static void cn0352_task_ADC_SYNC_START(void);
//info
static void cn0352_run_rb_validchsacn(void);
static void cn0352_run_rb_volt(BATTERY_T *ptr);
static void cn0352_run_rb_curr(BATTERY_T *ptr);
static void cn0352_run_rb_temp(BATTERY_T *ptr);
static void cn0352_run_rb_temp_sync(BATTERY_T *ptr);
static void cn0352_run_rb_vdac(BATTERY_T *ptr);
static void cn0352_run_rb_status(CHANNEL_T *ptr);
static void cn0352_run_rb_monitor(void);
static void cn0352_task_test_process_control(void);

static void cn0352_test_start_ch(CHANNEL_T *ptr);
static void cn0352_test_start_all(void);
static void cn0352_test_stop_ch(CHANNEL_T *ptr);
static void  cn0352_test_stop_all(void);

static float cn0352_fan_ctrl(void);
	
//Task Operation Reference
static void cn0352_task_Adp5065_RB(void);
static void cn0352_task_Adp5065_WR(void);


#define set_upd_bit(a,n)  (a[(n/(8*sizeof(a[0])))]|=(0x01<<(n%(8*sizeof(a[0])))))
#define clr_upd_bit(a,n)  (a[(n/(8*sizeof(a[0])))]&=(~(0x01<<(n%(8*sizeof(a[0]))))))
#define get_upd_bit(a,n) ((a[(n/(8*sizeof(a[0])))]&(0x01<<(n%(8*sizeof(a[0])))))? 0x1:0x0)



CN0352_T cn0352 = {		//CN0352 Initial
//	.dchg_term_volt   = {2.0  ,2.1 ,2.2  ,2.3  ,2.4  ,2.5  , 2.6 ,  2.7,  2.8,  2.9,  3.0,  3.1,  3.2,  3.3,  3.4,  3.5,  3.6,  3.7,  3.8,  3.9,  4.0,  4.1,  4.2},
	.dchg_term_volt     = {4.2  ,4.1 ,4.0  ,3.9  ,3.8  ,3.7  , 3.6 ,  3.5,  3.4,  3.3,  3.2,  3.1,  3.0,  2.9,  2.8,  2.7,  2.6,  2.5,  2.4,  2.3,  2.2,  2.1,  2.0},
	.dchg_term_curr     = {  0.0, 50.0,100.0,150.0,200.0,250.0,300.0,350.0,400.0,450.0,500.0,550.0,600.0,650.0,700.0,750.0,800.0,850.0,900.0,950.0,1000.0},
	.dchg_term_raw_curr = {0x000,(0x0AB>>1),(0x155>>1),(0x200>>1),(0x2AA>>1),(0x355>>1),(0x400>>1),(0x4AA>>1),(0x555>>1),(0x600>>1),(0x6AA>>1),(0x755>>1),(0x800>>1),(0x8AA>>1),(0x955>>1),(0x9FF>>1),(0xAAA>>1),(0xB55>>1),(0xBFF>>1),(0xCAA>>1),(0xD55>>1)},
	.name 		= 0x0352,
	.FirmVer 	= 0xA101,
	.ModuleID = 0xFF,
	.test_start = 0,
	.validch = {				//ValidCH
			.qty = 0,
			.flag.val = 0,
			.index = {0,0,0,0,0,0,0,0}
	},
	.adc_odr_index = ADC_UPDATE_INDEX_INIT,		//50Hz
	.baudrate = UART_BAUDRATE_INIT,
	.CHNO_BAT_Vdis = 0,
	.CHNO_BAT_Value = 0,
	.CHNO_ADP5065_RB = 0,
	.CHNO_ADP5065_WR = 0,
	.INDEX_ADP5065_REG_RB = 0,
	.INDEX_ADP5065_REG_WR = 0,
	.test_cycle_cnt = 0,
	.test_cycle = 10,
	.ch  = {						//Channel Iniital
		{						//CH0
			.ch_no = 0,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 0,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},
		{						//CH1
			.ch_no = 1,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},
		{						//CH2
			.ch_no = 2,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
			.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},
		{						//CH3
			.ch_no = 3,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},
		{						//CH4
			.ch_no = 4,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},
		{						//CH5
			.ch_no = 5,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},
		{						//CH6
			.ch_no = 6,
			.active = 0,
			.status = CHG_IDLE,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		},		
		{						//CH7
			.ch_no = 7,
			.active = 0,
//			.status = CHG_IDLE
			.status = CHG_Complete,
			.mstatus.Val = 0,
			.fault_count = 0,
			.test_cycle_cnt = 1,
			.cfg = {
				.InCurrIimt = I1000mA,
				.ChargeTricVth = V2P4V,
				.ChargeTricCurr = I20mA,
				.ChargeFastCurr = I850mA,
				.ChargeTermVolt = V4P20V,
				.ChargeTermCurr = I52P5mA,
				.DischargeTermVolt = {DV4P0, DV3P9, DV3P5},
				.DischargeTermCurr = {DI500mA, DI300mA, DI200mA}
			},
			.bat = {0},
			.adp5065w = {
				.reg.FuncSet1.Val = 0x4E		//Disable Charging
			},
			.adp5065r = {0},
			.adg715w = {0},
			.adg715r = {0}
		}
	},      //CH Initial Finished
	.run = {
		.cfg = {
			//Initialization
			.init = cn0352_run_cfg_init,
			.adp5065_init = cn0352_run_cfg_adp5065_init,
			.baudrateupd = cn0352_run_cfg_baudrateupd,
			.moduleIDupd = cn0352_run_cfg_moduleIDupd,
			// Charging Parameter Configuration			
			.InCurrLimit = cn0352_run_cfg_InCurrLimit,
			.TrickleVth = cn0352_run_cfg_TrickleVth,
			.TrickleCurr = cn0352_run_cfg_TrickleCurr,
			.FastCurr = cn0352_run_cfg_FastCurr,
			.TermVolt = cn0352_run_cfg_TermVolt,
			.TermCurr = cn0352_run_cfg_TermCurr,
			.DisTermVolt = cn0352_run_cfg_DisTermVolt,
			.DisTermCurr = cn0352_run_cfg_DisTermCurr,
			.start_volt_curr = cn0352_run_cfg_start_volt_curr,
			.start_temp_vdac = cn0352_run_cfg_start_temp_vdac,
			.start_temp_sync = cn0352_run_cfg_start_temp_sync,
			.Vdisc_Refresh = cn0352_task_Vdisc_Refresh,
			.Adp5065_RB = cn0352_task_Adp5065_RB,
			.Adp5065_WR = cn0352_task_Adp5065_WR,
			.CHGStart = cn0352_run_cfg_CHGStart,
			.CHGStop = cn0352_run_cfg_CHGStop,
			.DHGStart = cn0352_run_cfg_DHGStart,
			.DHGStop = cn0352_run_cfg_DHGStop,
			.sw_reset = cn0352_run_cfg_sw_reset,
			.InCurrLimit_Allset = cn0352_run_cfg_InCurrLimit_Allset,
			.TrickleVth_Allset = cn0352_run_cfg_TrickleVth_Allset,
			.TrickleCurr_Allset = cn0352_run_cfg_TrickleCurr_Allset,
			.FastCurr_Allset = cn0352_run_cfg_FastCurr_Allset,
			.TermVolt_Allset = cn0352_run_cfg_TermVolt_Allset,
			.TermCurr_Allset = cn0352_run_cfg_TermCurr_Allset,
			.DisTermVolt_Allset = cn0352_run_cfg_DisTermVolt_Allset,
			.DisTermCurr_Allset = cn0352_run_cfg_DisTermCurr_Allset,
			.test_cycle = cn0352_run_cfg_test_cycle,
			.CHG_CFG_ALL = cn0352_run_cfg_CHG_CFG_ALL,
			.CHGStart_Allset = cn0352_run_cfg_CHGStart_Allset,
			.CHGStop_Allset = cn0352_run_cfg_CHGStop_Allset,
			.adp5065_falut_recover = cn0352_run_cfg_adp5065_falut_recover,
			.fan_ctrl = cn0352_fan_ctrl
		},
		.test = {
			.start_ch = cn0352_test_start_ch,
			.start_all = cn0352_test_start_all,
			.stop_ch = cn0352_test_stop_ch,
			.stop_all = cn0352_test_stop_all,
		},
		.rb = {
			.validchsacn = cn0352_run_rb_validchsacn,
			.volt = cn0352_run_rb_volt,
			.curr = cn0352_run_rb_curr,
			.temp = cn0352_run_rb_temp,
			.temp_sync = cn0352_run_rb_temp_sync,
			.vdac = cn0352_run_rb_vdac,
			.status = cn0352_run_rb_status,
			.monitor = cn0352_run_rb_monitor
		}
	}
};

//CN0352_MODU_INFO_T cn0352_module_info = {
//	.p[0] = (unsigned char*)&cn0352.name,
//	.p[1] = (unsigned char*)&cn0352.name + 1,
//	.length = 2 + 2 + sizeof(ValidCH_T)
//};


// Task Definition		
//1st Refresh the Discharge Voltage for Active Channel
	static TASK_T task_DsichargeVoltageRefresh;
//2nd ADP5065 Register Read back
	static TASK_T task_ADP5065_REG_RB;
//3rd ADP5065 Register Writing
	static TASK_T task_ADP5065_REG_WR;
//4th Battery Voltage Current Read Back and Config for Temperature Measurement
	static TASK_T task_BAT_V_I_RB;
//6th Start Conversion
	static TASK_T task_ADC_SYNC_START_1;
//6th Battery Temperature , Discharge Voltage Readback
	static TASK_T task_BAT_T_VDIS_RB;
//7th Start Conversion
	static TASK_T task_ADC_SYNC_START_2;
//8tn cn0352_task_test_process_control
	static TASK_T task_test_process_control;

//Funstions Definition
//Initialization
static void cn0352_run_cfg_init(){
	unsigned char temp_ch = 0;
	float fadc_odr = 0.0;
	unsigned short count_task_BAT_V_I_RB = 0;
//Hardware Pin definition	
	pin_define();
//Clock, Watchdog Initialization
  WdtCfg(T3CON_PRE_DIV1,T3CON_IRQ_EN,T3CON_PD_DIS);  // Disable Watchdog timer resets
	//ClkDis(CLKDIS_DISSPI0CLK|CLKDIS_DISSPI1CLK|CLKDIS_DISDMACLK);
	ClkDis(CLKDIS_DISSPI0CLK|CLKDIS_DISSPI1CLK);
  ClkCfg(CLK_CD0,CLK_HF,CLKSYSDIV_DIV2EN_DIS,CLK_UCLKCG);            // Select CD0 for CPU clock - 16Mhz clock
  ClkSel(CLK_CD7,CLK_CD0,CLK_CD0,CLK_CD0);     // Select 16M for I2C System clk,16M for UART System clock	2M for PWM//int ClkSel(int iSpiCd, int iI2cCd, int iUrtCd, int iPwmCd) 	
//Interrupt Priority Configuration
//	NVIC_SetPriorityGrouping(6);	//Set Priority Group to 6			1bit PreemptPriority 2bit SubPriority
//	NVIC_SetPriority(TIMER0_IRQn,NVIC_EncodePriority(6,0,0));		//Timer0
//	NVIC_SetPriority(  UART_IRQn,NVIC_EncodePriority(6,1,0));		//Uart
//	NVIC_SetPriority(  ADC1_IRQn,NVIC_EncodePriority(6,1,1));		//ADC1
	NVIC_SetPriorityGrouping(6);	//Set Priority Group to 6			1bit PreemptPriority 2bit SubPriority
	NVIC_SetPriority(  UART_IRQn,NVIC_EncodePriority(6,0,0));		//UART_IRQn
	NVIC_SetPriority(TIMER0_IRQn,NVIC_EncodePriority(6,1,0));		//Timer0
	NVIC_SetPriority(  ADC1_IRQn,NVIC_EncodePriority(6,1,1));		//ADC1	
//Device Initialization	
	uart.init();
	//uart.dmainit();
	uart.setbaudrate(cn0352.baudrate);
	timer1.init();
	i2c.init();
	dac.init();
	adc.odr_tsettle(adc.ADCFLT_VALUE[cn0352.adc_odr_index],&fadc_odr,&cn0352.tsettle);
	timer0.init(TIMER0_LOAD_INIT);
	timer0.timeout(TIMER0_TIMEOUT_TICK);	
//Reset the Local Devices
	ctrlport.reset_on();//
	ctrlport.reset_off();//pCN0352_ctrlpout->BITs.IORST = 1;
//Initial the PWM signal
	pwm.init();
//Initializing the Valid Channel
	//Scan to find all valid Channels
	cn0352.run.rb.validchsacn();
	//ADP5065 Initialization
	cn0352.run.cfg.adp5065_init();
	//Disable Charging for all valid Channels
	for(temp_ch = 0; temp_ch < 8; temp_ch++){
		if(cn0352.ch[temp_ch].active == 1){
			cn0352.run.cfg.CHGStop(&cn0352.ch[temp_ch]);
		}
	}
	//Initial the counting channel number.
	for(temp_ch = 0; temp_ch < 8; temp_ch++){
		if(cn0352.ch[temp_ch].active == 1){
			cn0352.CHNO_ADP5065_RB = temp_ch;
			cn0352.CHNO_ADP5065_WR = temp_ch;
			cn0352.CHNO_BAT_Value = temp_ch;
			cn0352.CHNO_BAT_Vdis = temp_ch;
			break;
		}
	}
	cn0352.ModuleID = (unsigned char)(*ADDR_MODULE_INDEX);
//Periodic Task in Timer0 Interrupt Service Function Initial
//	void (*init)(TASK_T *task, unsigned int	InitialDelay, unsigned int	FirstPace, unsigned int	cycle, void(*operation)(void));	

	count_task_BAT_V_I_RB = (unsigned short)(cn0352.tsettle*((float)TIMER0_TIMEOUT_TICK));		//There is some error between real time with calculation time.
	count_task_BAT_V_I_RB += 2;																																//Try to find the closest number that could guarantee ADC conversion complete and satisfy the timing requirement
	while(count_task_BAT_V_I_RB%MIN_TASK_CYCLE){count_task_BAT_V_I_RB++;}																
	cn0352.samplerate = ((float)TIMER0_TIMEOUT_TICK/(float)((count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE)*2));

	alltask.init(&task_DsichargeVoltageRefresh, 0,   0,																					1*MIN_TASK_CYCLE,					                      cn0352_task_Vdisc_Refresh);
	alltask.init(					&task_ADP5065_REG_RB, 0,   1+0*MIN_TASK_CYCLE,												2*MIN_TASK_CYCLE,					                      cn0352_task_Adp5065_RB);
	alltask.init(					&task_ADP5065_REG_WR, 0,   1+1*MIN_TASK_CYCLE,												2*MIN_TASK_CYCLE,					                      cn0352_task_Adp5065_WR);
	
	alltask.init(							&task_BAT_V_I_RB, 0,  (2+1*count_task_BAT_V_I_RB), 									((count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE)*2), cn0352_task_BAT_V_I_RB);
	alltask.init(				&task_ADC_SYNC_START_1, 0,  (2+1*count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE), ((count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE)*2), cn0352_task_ADC_SYNC_START);
	
	//alltask.init(				 &task_BAT_T_VDIS_RB, 0, 	(2+2*count_task_BAT_V_I_RB), 						(count_task_BAT_V_I_RB*2), cn0352_task_BAT_T_VDIS_RB);	
	alltask.init(					 &task_BAT_T_VDIS_RB, 0, 	(2+2*count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE), ((count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE)*2), cn0352_task_BAT_T_SYNC_RB);
	alltask.init(				&task_ADC_SYNC_START_2, 0, 	(2+2*count_task_BAT_V_I_RB+2*MIN_TASK_CYCLE), ((count_task_BAT_V_I_RB+1*MIN_TASK_CYCLE)*2), cn0352_task_ADC_SYNC_START);
	alltask.init(		&task_test_process_control, 0,  3,                                          1*MIN_TASK_CYCLE,                               cn0352_task_test_process_control);
//Enable All Interrupt
	NVIC_EnableIRQ(UART_IRQn);
	NVIC_EnableIRQ(TIMER1_IRQn);
	NVIC_EnableIRQ(TIMER0_IRQn);
	NVIC_EnableIRQ(FLASH_IRQn);
	//NVIC_EnableIRQ(DMA_UART_TX_IRQn);                  // Enable UART Tx DMA interrupts
	adc.iext_curr_on();
	cn0352_run_cfg_adp5065_falut_recover();
	cn0352_run_cfg_start_volt_curr();					 // Start ADC
	//Enable timer0
	timer0.enable();
};

static void cn0352_run_cfg_baudrateupd(void){
	uart.init();
	uart.setbaudrate(cn0352.baudrate);
	timer1.init();
}


static unsigned char cn0352_run_cfg_moduleIDupd(void){
	unsigned long FlashData = 0;
	
	flash.erase_page(UserFlashPage);
	//ErasePage(UserFlashPage);
	FlashData = (unsigned long)cn0352.ModuleID;
	flash.wr(&FlashData,UserFlashPage,4);
	FlashData = *ADDR_MODULE_INDEX;
	if(FlashData == (unsigned long)cn0352.ModuleID){
		return(1);
	}
	return(0);
}

//cfg
static void cn0352_run_cfg_InCurrLimit(CHANNEL_T *ptr, unsigned char data){
	ptr->adp5065w.reg.VINxSet.BITs.ILIM = (data & 0x0F);
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_VINXSET);
};
static void cn0352_run_cfg_TrickleVth	(CHANNEL_T *ptr, unsigned char data){
	ptr->adp5065w.reg.VoltThr.BITs.VTRK_DEAD = (data & 0x03);
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_VOLTTHR);	
};
static void cn0352_run_cfg_TrickleCurr(CHANNEL_T *ptr, unsigned char data){
	ptr->adp5065w.reg.ChargeCur.BITs.ITRK_DEAD = (data & 0x03);
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_CHARGECUR);		
};
static void cn0352_run_cfg_FastCurr		(CHANNEL_T *ptr, unsigned char data){
	ptr->adp5065w.reg.ChargeCur.BITs.ICHG = (data & 0x07);
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_CHARGECUR);		
};
static void cn0352_run_cfg_TermVolt		(CHANNEL_T *ptr, unsigned char data){
	ptr->adp5065w.reg.TermSet.BITs.VTRM = (data & 0x3F);
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_TERMSET);		
};
static void cn0352_run_cfg_TermCurr		(CHANNEL_T *ptr, unsigned char data){
	ptr->adp5065w.reg.TermSet.BITs.IEND = (data & 0x03);
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_TERMSET);
};
static void cn0352_run_cfg_DisTermVolt(CHANNEL_T *ptr, unsigned char data[3]){
	ptr->cfg.DischargeTermVolt[0] = (DISCHARGE_TERM_VOLT_T)data[0];			//Discharge Voltage need to converted to int and then writted into the register.
	ptr->cfg.DischargeTermVolt[1] = (DISCHARGE_TERM_VOLT_T)data[1];			//Discharge Voltage need to converted to int and then writted into the register.
	ptr->cfg.DischargeTermVolt[2] = (DISCHARGE_TERM_VOLT_T)data[2];			//Discharge Voltage need to converted to int and then writted into the register.
};
static void cn0352_run_cfg_DisTermCurr(CHANNEL_T *ptr, unsigned char data[3]){
	ptr->cfg.DischargeTermCurr[0] = (DISCHARGE_TERM_CURR_T)data[0];			//Discharge Voltage need to converted to int and then writted into the register.
	ptr->cfg.DischargeTermCurr[1] = (DISCHARGE_TERM_CURR_T)data[1];			//Discharge Voltage need to converted to int and then writted into the register.
	ptr->cfg.DischargeTermCurr[2] = (DISCHARGE_TERM_CURR_T)data[2];			//Discharge Voltage need to converted to int and then writted into the register.	
};
static void cn0352_run_cfg_CHGStart(CHANNEL_T *ptr){
	if(ptr->adp5065r.reg.FuncSet1.BITs.EN_CHG == 0){		//Now Stoped ?
		ptr->adp5065w.reg.FuncSet1.BITs.EN_CHG = 1;							//Start!
		set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_FUNCSET1);
	}
};
static void cn0352_run_cfg_CHGStop(CHANNEL_T *ptr){
	if(ptr->adp5065r.reg.FuncSet1.BITs.EN_CHG == 1){		//Now Starting ?
		ptr->adp5065w.reg.FuncSet1.BITs.EN_CHG = 0;							//Stop!
		set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_FUNCSET1);
	}
};
static void cn0352_run_cfg_DHGStart(CHANNEL_T *ptr){
	//1st Disable Charging
	ptr->adp5065w.reg.FuncSet1.BITs.EN_CHG = 0;
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_FUNCSET1);
	//2nd Set the DisCharging Current
	//3rd Set Charging Status to Discharge_Step1
	ptr->status =	DCHG_S1;
};
static void cn0352_run_cfg_DHGStop(CHANNEL_T *ptr){
	ptr->adp5065w.reg.FuncSet1.BITs.EN_CHG = 0;
	set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_FUNCSET1);	
	ptr->status =	CHG_IDLE;
};
static void cn0352_run_cfg_start_volt_curr(void){
//	adc.iext_curr_off();	
	adc.init_bat_volt_curr(cn0352.adc_odr_index);
	adc.sync_start();	
};
static void cn0352_run_cfg_start_temp_vdac(void){
	adc.init_bat_temp_vdac(cn0352.adc_odr_index);
	adc.sync_start();
};
static void cn0352_run_cfg_start_temp_sync(void){
//	adc.iext_curr_on();
	adc.init_bat_temp_sync(cn0352.adc_odr_index);
	adc.sync_start();
};

static void cn0352_run_cfg_sw_reset(void){
	SCB->AIRCR = 0x05FA0004;		//Softwawre Reset.
}
//info
static void cn0352_run_rb_validchsacn(void){
	unsigned char ch_no = 0;
	unsigned int IntStatus0 = 0;
	unsigned int IntStatus1 = 0;
	cn0352.validch.qty = 0;
	cn0352.validch.flag.val = 0;
	//Store the Current Interrupt Configuration
	IntStatus0 = NVIC->ISER[0];
	IntStatus1 = NVIC->ISER[1];
	//Disable Timer0 Interrupt
//	NVIC_DisableIRQ(TIMER0_IRQn);
	ctrlport.i2c_open();
	for(ch_no = 0; ch_no < 8; ch_no ++){
		cn0352.ch[ch_no].active = 0;
		ctrlport.set_ch(ch_no);
		if(i2c.nackaddr(I2CID_ADG715)){
			cn0352.validch.index[cn0352.validch.qty] = ch_no;
			cn0352.validch.qty++;
			cn0352.validch.flag.val |=(1<<ch_no);
			cn0352.ch[ch_no].active = 1;
		}
	}
	//Recover the Interrupt Status
	NVIC->ISER[0] = IntStatus0;
	NVIC->ISER[1] = IntStatus1;
	//Enable Timer0 Interrupt
//	NVIC_EnableIRQ(TIMER0_IRQn);	
};
static void cn0352_run_rb_volt(BATTERY_T *ptr){
	uint32_t utemp = 0;
#ifdef ENABLE_CAL_FLOAT_RESULT		
	int32_t temp = 0;
#endif	
	utemp = adc.raw_ch0();
	temp = (int32_t)utemp;
	ptr->raw_volt = utemp;
#ifdef ENABLE_CAL_FLOAT_RESULT	
	ptr->volt = 0.000000022351741790771484375*((float)temp);	//0.000000022351741790771484375 = 5*1.2/2exp(28)
#endif	
};
static void cn0352_run_rb_curr(BATTERY_T *ptr){
	uint32_t utemp = 0;
#ifdef ENABLE_CAL_FLOAT_RESULT		
	int32_t temp = 0;
#endif	
	utemp = adc.raw_ch1();	
	ptr->raw_curr = utemp;
	temp = (int32_t)utemp;
#ifdef ENABLE_CAL_FLOAT_RESULT		
	ptr->curr = 0.000000022351741790771484375*((float)temp);	//Vadc = 10*I*Rcs = 200mOhm*I ==> I = Vadc(V)*5/Ohm = 5*Vadc(A) = 5*1.2/2exp(28)*Data = 0.000000022351741790771484375*Data. 0.000000022351741790771484375 = 5*1.2/2exp(28)
#endif	
};
static void cn0352_run_rb_temp(BATTERY_T *ptr){
	ptr->raw_temp = adc.raw_ch1();
};
static void cn0352_run_rb_temp_sync(BATTERY_T *ptr){
	uint32_t utemp = 0;
	uint32_t utempref = 0;
#ifdef ENABLE_CAL_FLOAT_RESULT		
	int32_t temp = 0;	
	int32_t tempref = 0;	
	float resistance;
	#define resistance_ref		2.2
#endif	
	utemp    = adc.raw_ch0();
	utempref = adc.raw_ch1();
	ptr->raw_temp    = utemp;
	ptr->raw_tempref = utempref;
#ifdef ENABLE_CAL_FLOAT_RESULT		
	temp 		= (int32_t)utemp;
	tempref = (int32_t)utempref;
	resistance = (((float)temp*resistance_ref)/(float)tempref);
	ptr->temp = ERTJZEG103JA.getTemp(resistance);
	//ptr->temp 		= 0.000000004470348358154296875*((float)temp);	//0.000000004470348358154296875 = 1.2/2exp(28)
	//ptr->tempref	= 0.000000004470348358154296875*((float)tempref);	//0.000000004470348358154296875 = 1.2/2exp(28)
	//ptr->temp 		= 0.00000001229345798492431640625*((float)temp);	//0.00000001229345798492431640625 = 3.3/2exp(28)
	ptr->tempref	= 0.00000001229345798492431640625*((float)tempref);	//0.00000001229345798492431640625 = 3.3/2exp(28)	

#endif
};
static void cn0352_run_rb_vdac(BATTERY_T *ptr){
	ptr->raw_vdisrb = adc.raw_ch0();
};
static void cn0352_run_rb_status(CHANNEL_T *ptr){
};
static void cn0352_run_rb_monitor(void){
	unsigned char chno = 0;
	for(chno = 0; chno < 8 ; chno ++){
		if((cn0352.ch[chno].active == 1)&&(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 1)){
			if(monitor.charging(&cn0352.ch[chno].adp5065r)){			//Charge Issue Happened.
				cn0352.run.cfg.CHGStop(&cn0352.ch[chno]);
			}
		}
	}
}
//Task
static void cn0352_task_Vdisc_Refresh(void){
	unsigned char temp_ch;
	
	if(cn0352.validch.qty >0){
		//1st Store Current Control Port Value.
		temp_ch = ctrlport.get_ch();
		//2nd Select the Channel that DAC just updated.
		ctrlport.set_ch(cn0352.CHNO_BAT_Vdis);							
		//3rd Turn off the DAC Channel
		cn0352.ch[cn0352.CHNO_BAT_Vdis].adg715w.FUNC.VDAC = 0;	
		adg715.wr(cn0352.ch[cn0352.CHNO_BAT_Vdis].adg715w.Val);
		//4th Search the Next Valid Channel
		do{
			cn0352.CHNO_BAT_Vdis = (cn0352.CHNO_BAT_Vdis == 7)? 0:(cn0352.CHNO_BAT_Vdis+1);
		}while(!cn0352.ch[cn0352.CHNO_BAT_Vdis].active);	//if active = 0, switch to the next channel.
		//5th Select the Channel that DAC need to update 
			ctrlport.set_ch(cn0352.CHNO_BAT_Vdis);	
		//6th Updated the DAC Value for the channels that are in Discharging Process
		if(cn0352.ch[cn0352.CHNO_BAT_Vdis].status == DCHG_S1){
			//dac.wr_raw12(cn0352.ch[cn0352.CHNO_BAT_Vdis].bat.raw_vdis);
			dac.wr_raw12(cn0352.dchg_term_raw_curr[cn0352.ch[cn0352.CHNO_BAT_Vdis].cfg.DischargeTermCurr[0]]);
		}
		else if(cn0352.ch[cn0352.CHNO_BAT_Vdis].status == DCHG_S2){
			dac.wr_raw12(cn0352.dchg_term_raw_curr[cn0352.ch[cn0352.CHNO_BAT_Vdis].cfg.DischargeTermCurr[1]]);
		}
		else if(cn0352.ch[cn0352.CHNO_BAT_Vdis].status == DCHG_S3){
			dac.wr_raw12(cn0352.dchg_term_raw_curr[cn0352.ch[cn0352.CHNO_BAT_Vdis].cfg.DischargeTermCurr[2]]);
		}
		else{
			dac.wr_raw12(0);	//Other Channels in charging mode or in idle status will be shut down the discharging current.
		}
		//7th Turn On the DAC Channel
		cn0352.ch[cn0352.CHNO_BAT_Vdis].adg715w.FUNC.VDAC = 1;
		adg715.wr(cn0352.ch[cn0352.CHNO_BAT_Vdis].adg715w.Val);
		//Recover the previous Channel
		ctrlport.set_ch(temp_ch);	
	}
};
static void cn0352_task_Adp5065_RB(void){
	unsigned char temp_ch;
	if(cn0352.validch.qty >0){	
		//1st Store Current Control Port Value.
		temp_ch = ctrlport.get_ch();
		ctrlport.set_ch(cn0352.CHNO_ADP5065_RB);
		//2nd Read ADP5065 Register of the specific channel in sequence
		//if(cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r_fupd == 0){	//hasn't read by monitor program.
			cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.array[adp5065.id_readable[cn0352.INDEX_ADP5065_REG_RB]] = adp5065.rd(adp5065.id_readable[cn0352.INDEX_ADP5065_REG_RB]);
			if(!get_upd_bit(cn0352.ch[cn0352.CHNO_ADP5065_RB].fw_adg5065, adp5065.id_writable[cn0352.INDEX_ADP5065_REG_RB])){	//If there is no Updated Value from PC, updated the writable value to readback value from ADP5065
				cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065w.array[adp5065.id_readable[cn0352.INDEX_ADP5065_REG_RB]] = cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.array[adp5065.id_readable[cn0352.INDEX_ADP5065_REG_RB]];
			}
		//}
		
		//3th Update the Channel Number and Register Number
		if(cn0352.INDEX_ADP5065_REG_RB == SIZE_ADP5065_READABLE_REG-1){
			cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r_fupd = 1;
			cn0352.INDEX_ADP5065_REG_RB = 0;
			//Begin of Update Battery Monitoring Status.
			cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.VIN_STA = (cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta1.BITs.VIN_OV + (cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta1.BITs.VIN_OK<<1));
			cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.IIN_LIM = cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta1.BITs.VIN_ILIM;
			if(cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta1.BITs.THERM_LIM == 0){
				cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.TSD_STA = 0x0;
			}
			else if(cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.FaultReg.BITs.TSDL_130 == 0){
				cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.TSD_STA = 0x1;
			}
			else if(cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.FaultReg.BITs.TSD_140 == 0){
				cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.TSD_STA = 0x2;
			}
			else{
				cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.TSD_STA = 0x3;
			}
			cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.IPK_STA = cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta2.BITs.IPK_STAT;
			//Battery Voltage Monitoring Redefine.
			cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_SHR = cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.FaultReg.BITs.BAT_SHR;
			cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_VSTA = (unsigned short)cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta2.BITs.BATTERY_STATUS;
			//Battery Temperature Monitoring Redefine.
			switch(cn0352.ch[cn0352.CHNO_ADP5065_RB].adp5065r.reg.ChargerSta2.BITs.THR_STATUS){
				case 0:{	//000 = battery temperature monitoring off.
					cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_TSTA = 0;
					break;
				}
				case 1:{	//001 = battery cold
					cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_TSTA = 1;
					break;
				}
				case 2:{	//010 = battery cool. 
					cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_TSTA = 2;
					break;
				}
				case 3:{	//011 = battery warm. 
					cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_TSTA = 4;
					break;
				}
				case 4:{	//100 = battery hot. 
					cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_TSTA = 5;
					break;
				}
				case 7:{	//111 = thermistor OK.
					cn0352.ch[cn0352.CHNO_ADP5065_RB].mstatus.REG.BAT_TSTA = 3;
					break;
				}
				default:{	//Reserved
					break;
				}
			}
			//End of Update Battery Monitoring Status.
			//Update Channel Number to the Next Valid Channel Number
			do{
				cn0352.CHNO_ADP5065_RB = (cn0352.CHNO_ADP5065_RB == 7)? 0:(cn0352.CHNO_ADP5065_RB+1);
			}while(!cn0352.ch[cn0352.CHNO_ADP5065_RB].active);	//if active = 0, switch to the next channel.	
		}
		else
			cn0352.INDEX_ADP5065_REG_RB ++;
		//4th Recover the Port
		ctrlport.set_ch(temp_ch);
	}
};
static void cn0352_task_Adp5065_WR(void){
	unsigned char temp_ch;
	if(cn0352.validch.qty >0){	
		//1st Store Current Control Port Value.
		temp_ch = ctrlport.get_ch();
		ctrlport.set_ch(cn0352.CHNO_ADP5065_WR);
		//2nd Write ADP5065 Register of the specific channel in sequence
		for(cn0352.INDEX_ADP5065_REG_WR = 0; cn0352.INDEX_ADP5065_REG_WR < SIZE_ADP5065_WRITABLE_REG; cn0352.INDEX_ADP5065_REG_WR ++){
			if(get_upd_bit(cn0352.ch[cn0352.CHNO_ADP5065_WR].fw_adg5065, adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR])){
				adp5065.wr(adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR],cn0352.ch[cn0352.CHNO_ADP5065_WR].adp5065w.array[adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR]]);
				clr_upd_bit(cn0352.ch[cn0352.CHNO_ADP5065_WR].fw_adg5065, adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR]);
				break;				
			}
			else if(adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR] == ADP5065_REGID_TIMERSET){
				adp5065.wr(adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR],(cn0352.ch[cn0352.CHNO_ADP5065_WR].adp5065w.array[adp5065.id_writable[cn0352.INDEX_ADP5065_REG_WR]]|0x01));//Clear Watch Dog Timer
			}
		}
		//3th Update the Channel Number and Register Number
		if(cn0352.INDEX_ADP5065_REG_WR == SIZE_ADP5065_WRITABLE_REG){
			cn0352.INDEX_ADP5065_REG_WR = 0;
			do{
				cn0352.CHNO_ADP5065_WR = (cn0352.CHNO_ADP5065_WR == 7)? 0:(cn0352.CHNO_ADP5065_WR+1);
			}while(!cn0352.ch[cn0352.CHNO_ADP5065_WR].active);	//if active = 0, switch to the next channel.			
		}
		//4th Recover the Port
		ctrlport.set_ch(temp_ch);
	}
};
static void cn0352_task_BAT_V_I_RB(void){
	if(cn0352.validch.qty >0){
		cn0352_run_rb_volt(&cn0352.ch[cn0352.CHNO_BAT_Value].bat);
		cn0352_run_rb_curr(&cn0352.ch[cn0352.CHNO_BAT_Value].bat);
		adc.init_bat_temp_sync(cn0352.adc_odr_index);
	}
}

static void cn0352_task_BAT_T_VDIS_RB(void){
	unsigned char temp_ch;
	if(cn0352.validch.qty >0){
		//1st Read the Previous Conversion Value 
		cn0352_run_rb_temp(&cn0352.ch[cn0352.CHNO_BAT_Value].bat);
		cn0352_run_rb_vdac(&cn0352.ch[cn0352.CHNO_BAT_Value].bat);
		cn0352.ch[cn0352.CHNO_BAT_Value].bat.f_upd = 1;
		cn0352.ch[cn0352.CHNO_BAT_Value].bat.f_upd_process = 1;
		cn0352.ch[cn0352.CHNO_BAT_Value].count++;
		if(cn0352.test_start == 1){
			bat_info.wr();			//update bat_info buffer when testing started.
		}
		//2nd Store Current Control Port Value.
		temp_ch = ctrlport.get_ch();
		//3rd Select the Channel that DAC just updated.
		ctrlport.set_ch(cn0352.CHNO_BAT_Value);							
		//4th Turn off ALL the Analog Signal Routing to ADC
		cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.FUNC.Others = 0;	
		adg715.wr(cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.Val);
		//5th Search the Next Valid Channel
		do{
			cn0352.CHNO_BAT_Value = (cn0352.CHNO_BAT_Value == 7)? 0:(cn0352.CHNO_BAT_Value+1);
		}while(!cn0352.ch[cn0352.CHNO_BAT_Value].active);	//if active = 0, switch to the next channel.
		//6th Pointed to the Next Available Channel
		ctrlport.set_ch(cn0352.CHNO_BAT_Value);	
		//7th Turn On All the Analog Signals on the Next Available Channel into ADC
		cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.FUNC.Others = 0x7F;
		adg715.wr(cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.Val);
		//8th Start Conversion
		cn0352_run_cfg_start_volt_curr();
		//9th Recover the previous Channel
		ctrlport.set_ch(temp_ch);	
	}
}
static void cn0352_task_BAT_T_SYNC_RB(void){
	unsigned char ch_no = 0;
	if(cn0352.validch.qty >0){
		//1st Read the Previous Conversion Value 
		cn0352_run_rb_temp_sync(&cn0352.ch[cn0352.CHNO_BAT_Value].bat);
		cn0352.ch[cn0352.CHNO_BAT_Value].bat.f_upd = 1;
		cn0352.ch[cn0352.CHNO_BAT_Value].bat.f_upd_process = 1;
		cn0352.ch[cn0352.CHNO_BAT_Value].count++;
		if(cn0352.test_start == 1){
			if((cn0352.CHNO_BAT_Value == cn0352.validch.index[cn0352.validch.qty-1])&&
				((bat_info.buf.Wused.Val + cn0352.validch.qty) < BAT_INFO_BUF_LENGTH)){
				for(ch_no = 0; ch_no < cn0352.validch.qty; ch_no++){
					bat_info.wr();			//update bat_info buffer.
				}
			}
		}
		//2nd Store Current Control Port Value.
//		temp_ch = ctrlport.get_ch();
		//3rd Select the Channel that DAC just updated.
		ctrlport.set_ch(cn0352.CHNO_BAT_Value);							
		//4th Turn off ALL the Analog Signal Routing to ADC
		cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.FUNC.Others = 0;	
		adg715.wr(cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.Val);
		//5th Search the Next Valid Channel
		do{
			cn0352.CHNO_BAT_Value = (cn0352.CHNO_BAT_Value == 7)? 0:(cn0352.CHNO_BAT_Value+1);
		}while(!cn0352.ch[cn0352.CHNO_BAT_Value].active);	//if active = 0, switch to the next channel.
		//6th Pointed to the Next Available Channel
		ctrlport.set_ch(cn0352.CHNO_BAT_Value);	
		//7th Turn On All the Analog Signals on the Next Available Channel into ADC
		cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.FUNC.Others = 0xFF;
		adg715.wr(cn0352.ch[cn0352.CHNO_BAT_Value].adg715w.Val);
		//8th Start Conversion
		//cn0352_run_cfg_start_volt_curr();
		adc.init_bat_volt_curr(cn0352.adc_odr_index);
		//9th Recover the previous Channel
//		ctrlport.set_ch(temp_ch);	
	}		
}
static void cn0352_task_ADC_SYNC_START(void){
	adc.sync_start();
}
static void cn0352_task_test_process_control(void){
	unsigned char chno = 0;
	unsigned char temp_start = 0;
	if(cn0352.test_start == 1){				//PC CMD Start the test.
		if(cn0352.validch.qty > 0){
			for(chno = 0; chno < 8; chno ++){
				if(cn0352.ch[chno].active == 1){
					if(cn0352.ch[chno].test_start == 1){
						if(cn0352.ch[chno].adp5065r_fupd == 1){
							switch(cn0352.ch[chno].status){
								case CHG_IDLE:{			// Start Charging
									if(cn0352.ch[chno].test_cycle_cnt == cn0352.test_cycle){
										cn0352_run_cfg_CHGStop(&cn0352.ch[chno]);	//Disable Charging
										if(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 0){				//Has been stopped
											cn0352.ch[chno].bat.raw_vdis = 0;	//Disable Discharging Current.										
											cn0352.ch[chno].status = TEST_COMPLETE;
										}
									}
									else if(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 0){		//Cycle Number not reached and Previous cycle completed, start a new cycle
										cn0352_run_cfg_CHGStart(&cn0352.ch[chno]);
									}
									else{	//Cycle Number not reached and Current cycle is under going.
										cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									}
									break;
								}
								case CHG_TRICKLE:{
									cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									break;
								}
								case CHG_CC:{
									cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									break;
								}
								case CHG_CV:{
									cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									break;
								}
								case CHG_Complete:{
									if(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 1){
										cn0352_run_cfg_CHGStop(&cn0352.ch[chno]);
									}
									else{
										cn0352.ch[chno].status = DCHG_S1;
										cn0352.ch[chno].bat.raw_vdis = cn0352.dchg_term_raw_curr[cn0352.ch[chno].cfg.DischargeTermCurr[0]];	//update discharging current.
									}
									break;
								}
								case CHG_SUSPEND:{
									cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									if(cn0352.ch[chno].fault_count < ALLOWED_FAULT_NUMBER){	//Try to recover the fault for allowed times.
										cn0352_run_cft_adp5065_falut_recover_channel(&cn0352.ch[chno]);
										cn0352.ch[chno].fault_count++;
									}
									else{
										cn0352.ch[chno].test_start = 0; //Test failed.
									}
									break;
								}
								case CHG_TIMER_EXPIRED:{
									cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									if(cn0352.ch[chno].fault_count < ALLOWED_FAULT_NUMBER){	//Try to recover the fault for allowed times.
										cn0352_run_cft_adp5065_falut_recover_channel(&cn0352.ch[chno]);
										cn0352.ch[chno].fault_count++;
									}
									else{
										cn0352.ch[chno].test_start = 0; //Test failed.
									}
									break;
								}
								case CHG_BATTERY_DETECTION:{
									cn0352.ch[chno].status = (CHANNEL_STATUS_T)cn0352.ch[chno].adp5065r.reg.ChargerSta1.BITs.CHARGER_STATUS;
									break;								
								}
								case DCHG_S1:{
									if(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 1){		//Make sure the charger was closed during discharging
										cn0352_run_cfg_CHGStop(&cn0352.ch[chno]);
									}
									if(cn0352.ch[chno].bat.f_upd_process == 1){
									//if(1){
										if(cn0352.ch[chno].bat.volt <= cn0352.dchg_term_volt[cn0352.ch[chno].cfg.DischargeTermVolt[0]]){//Discharge Step 1 Complete.
											cn0352.ch[chno].status = DCHG_S2;
											cn0352.ch[chno].bat.raw_vdis = cn0352.dchg_term_raw_curr[cn0352.ch[chno].cfg.DischargeTermCurr[1]];	//update discharging current.
										}
										cn0352.ch[chno].bat.f_upd_process = 0;
									}
									break;
								}
								case DCHG_S2:{
									if(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 1){		//Make sure the charger was closed during discharging
										cn0352_run_cfg_CHGStop(&cn0352.ch[chno]);
									}								
									if(cn0352.ch[chno].bat.f_upd_process == 1){
									//if(1){	
										if(cn0352.ch[chno].bat.volt <= cn0352.dchg_term_volt[cn0352.ch[chno].cfg.DischargeTermVolt[1]]){																									//Discharge Step 2 Complete.
											cn0352.ch[chno].status = DCHG_S3;
											cn0352.ch[chno].bat.raw_vdis = cn0352.dchg_term_raw_curr[cn0352.ch[chno].cfg.DischargeTermCurr[2]];	//update discharging current.
										}
										cn0352.ch[chno].bat.f_upd_process = 0;
									}
									break;
								}		
								case DCHG_S3:{
									if(cn0352.ch[chno].adp5065r.reg.FuncSet1.BITs.EN_CHG == 1){		//Make sure the charger was closed during discharging
										cn0352_run_cfg_CHGStop(&cn0352.ch[chno]);
									}
									if(cn0352.ch[chno].bat.f_upd_process == 1){
									//if(1){	
										if(cn0352.ch[chno].bat.volt <= cn0352.dchg_term_volt[cn0352.ch[chno].cfg.DischargeTermVolt[2]]){																									//Discharge Step 3 Complete.
											cn0352.ch[chno].status = CHG_IDLE;
											cn0352.ch[chno].bat.raw_vdis = 0;	//Disable Discharging Current.
											cn0352.ch[chno].test_cycle_cnt++;
										}
										cn0352.ch[chno].bat.f_upd_process = 0;
									}
									break;
								}
								case TEST_COMPLETE:{
									cn0352.ch[chno].test_start = 0;
									break;
								}
								default:{
									break;
								}
							}
							cn0352.ch[chno].adp5065r_fupd = 0;
						}
					}
					temp_start += cn0352.ch[chno].test_start;
				}
			}
		}
		if(temp_start == 0){			//All test Comepelte
			cn0352.test_start = 0;	//Test Complete
		}
	}
}

//
static void cn0352_run_cfg_InCurrLimit_Allset(unsigned char data){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.InCurrIimt = (CHARGE_INCURRLIMIT_T)data;
			cn0352_run_cfg_InCurrLimit(&cn0352.ch[chno],data);
		}
	}
};
static void cn0352_run_cfg_TrickleVth_Allset(unsigned char data){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.ChargeTricVth = (CHARGE_TRIC_VTH_T)data;
			cn0352_run_cfg_TrickleVth(&cn0352.ch[chno],data);
		}
	}	
};
static void cn0352_run_cfg_TrickleCurr_Allset(unsigned char data){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.ChargeTricCurr = (CHARGE_TRIC_CURR_T)data;
			cn0352_run_cfg_TrickleCurr(&cn0352.ch[chno],data);
		}
	}		
};
static void cn0352_run_cfg_FastCurr_Allset(unsigned char data){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.ChargeFastCurr = (CHARGE_FAST_CURR_T)data;
			cn0352.run.cfg.FastCurr(&cn0352.ch[chno],data);
		}
	}		
};
static void cn0352_run_cfg_TermVolt_Allset(unsigned char data){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.ChargeTermVolt = (CHARGE_TERM_VOLT_T)data;
			cn0352_run_cfg_TermVolt(&cn0352.ch[chno],data);
		}
	}		
};
static void cn0352_run_cfg_TermCurr_Allset(unsigned char data){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.ChargeTermCurr = (CHARGE_TERM_CURR_T)data;
			cn0352_run_cfg_TermCurr(&cn0352.ch[chno],data);
		}
	}			
};
static void cn0352_run_cfg_DisTermVolt_Allset(unsigned char data[3]){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.DischargeTermVolt[0] = (DISCHARGE_TERM_VOLT_T)data[0];
			cn0352.ch[chno].cfg.DischargeTermVolt[1] = (DISCHARGE_TERM_VOLT_T)data[1];
			cn0352.ch[chno].cfg.DischargeTermVolt[2] = (DISCHARGE_TERM_VOLT_T)data[2];
			cn0352_run_cfg_DisTermVolt(&cn0352.ch[chno],data);
		}
	}		
};
static void cn0352_run_cfg_DisTermCurr_Allset(unsigned char data[3]){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].cfg.DischargeTermCurr[0] = (DISCHARGE_TERM_CURR_T)data[0];
			cn0352.ch[chno].cfg.DischargeTermCurr[1] = (DISCHARGE_TERM_CURR_T)data[1];
			cn0352.ch[chno].cfg.DischargeTermCurr[2] = (DISCHARGE_TERM_CURR_T)data[2];			
			cn0352_run_cfg_DisTermCurr(&cn0352.ch[chno],data);
		}
	}
};
static void cn0352_run_cfg_test_cycle(unsigned short data){
	unsigned char chno; 
	cn0352.test_cycle = data;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352.ch[chno].test_cycle_cnt = data;
		}
	}	
}
static void cn0352_run_cfg_CHG_CFG_ALL(FORMAT_CFG_ALL_T* cfg){
	cn0352_run_cfg_InCurrLimit_Allset(cfg->REG.InCurrLimit.data);
	cn0352_run_cfg_TrickleVth_Allset(cfg->REG.TrickleVth.data);
	cn0352_run_cfg_TrickleCurr_Allset(cfg->REG.TrickleCurr.data);
	cn0352_run_cfg_FastCurr_Allset(cfg->REG.FastCurr.data);
	cn0352_run_cfg_TermVolt_Allset(cfg->REG.TermVolt.data);
	cn0352_run_cfg_TermCurr_Allset(cfg->REG.TermCurr.data);
	cn0352_run_cfg_DisTermVolt_Allset(cfg->REG.DisTermVolt.data);
	cn0352_run_cfg_DisTermCurr_Allset(cfg->REG.DisTermCurr.data);
	cn0352_run_cfg_test_cycle(cfg->REG.TestCycle.data);
};
static void cn0352_run_cfg_CHGStart_Allset(void){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352_run_cfg_CHGStart(&cn0352.ch[chno]);
		}
	}
};
static void cn0352_run_cfg_CHGStop_Allset(void){
	unsigned char chno;
	for(chno = 0; chno <8; chno++){
		if(cn0352.ch[chno].active == 1){
			cn0352_run_cfg_CHGStop(&cn0352.ch[chno]);
		}
	}		
};
//static void cn0352_run_cfg_DHGStart_Allset(void){
//};
//static void cn0352_run_cfg_DHGStop_Allset(void){
//};
static void cn0352_run_cfg_adp5065_init(void){
unsigned char chno = 0;
unsigned char writtable_index = 0;
	for(chno = 0; chno < 8 ; chno ++){
		if(cn0352.ch[chno].active == 1){
			for(writtable_index = 0; writtable_index < SIZE_ADP5065_WRITABLE_REG; writtable_index++){
				cn0352.ch[chno].adp5065w.array[adp5065.id_writable[writtable_index]] = adp5065_default.array[adp5065.id_writable[writtable_index]];
				set_upd_bit(cn0352.ch[chno].fw_adg5065, adp5065.id_writable[writtable_index]);
			}
		}
	}
};
static void cn0352_run_cft_adp5065_falut_recover_channel(CHANNEL_T *ptr){
		if(ptr->active == 1){
			if(ptr->adp5065r.reg.FaultReg.Val){//Some Fault Condition Happened.
				ptr->adp5065w.reg.FaultReg.Val = ptr->adp5065r.reg.FaultReg.Val;	//Clear the related Fault Condition
				set_upd_bit(ptr->fw_adg5065, ADP5065_REGID_FAULTREG);
			}
		}	
}
static void cn0352_run_cfg_adp5065_falut_recover(void){
unsigned char chno = 0;
	for(chno = 0; chno < 8 ; chno ++){
		cn0352_run_cft_adp5065_falut_recover_channel(&cn0352.ch[chno]);
	}
}
static void cn0352_test_start_ch(CHANNEL_T *ptr){
	ptr->test_start = 1;
	ptr->test_cycle_cnt = 0;
	cn0352_run_cfg_CHGStart(ptr);
}
static void cn0352_test_start_all(void){
	unsigned char chno = 0;
	cn0352.test_cycle_cnt = 0;
	for(chno = 0; chno < 8 ; chno ++){
		if(cn0352.ch[chno].active == 1){
			cn0352_test_start_ch(&cn0352.ch[chno]);
		}
	}
	cn0352.test_start = 1;
};
static void cn0352_test_stop_ch(CHANNEL_T *ptr){
	ptr->test_start = 0;
	ptr->test_cycle_cnt = 0;
	cn0352_run_cfg_CHGStop(ptr);
};
static void cn0352_test_stop_all(void){
	unsigned char chno = 0;
	cn0352.test_cycle_cnt = 0;
	for(chno = 0; chno < 8 ; chno ++){
		if(cn0352.ch[chno].active == 1){
			cn0352_test_stop_ch(&cn0352.ch[chno]);
		}
	}
};
static float cn0352_fan_ctrl(void){
	//The FAN control prog is used to control the board temperature in order to protect the charger and other components on board.
	//two fans available on CN0352. Each fan would supposed for 4 channels(FAN1: CH0~3, FAN2: CH4~7
	//The temperature of battery would be sensed by external Thermistor or RTD and samplede by ADC of ADuCM360.
	//The thermistor 10K would be populated on board near the discharging circuit in order to sense the tempearture during discuaring.
	//FAN controlled loop will be feed by the status of THR_STATUS of ADP5065 and controlled roughly.
	//The tempearture threshold would be:
	//	THR_STATUS_BAT_COLD					below  0 deg C
	//	THR_STATUS_BAT_COOL					above  0 deg C
	//	THR_STATUS_BAT_WARM					above 45 deg C
	//	THR_STATUS_BAT_HOT					above 60 deg C
	//	THR_STATUS_BAT_OK						  0 ~ 45 deg C
	unsigned char chno = 0;
	enum {FAN_SLOW, FAN_NORMAL,FAN_FAST} FAN;
	float duty = 0.5;
	FAN = FAN_NORMAL;
	for(chno = 0; chno < 8; chno ++){
		if(cn0352.ch[chno].active == 1){
//			if(cn0352.ch[chno].adp5065r_fupd == 1){
					switch(FAN){
						case FAN_SLOW: {
							if((cn0352.ch[chno].adp5065r.reg.ChargerSta2.BITs.THR_STATUS == THR_STATUS_BAT_WARM) ||
								 (cn0352.ch[chno].adp5065r.reg.ChargerSta2.BITs.THR_STATUS == THR_STATUS_BAT_HOT)){
								FAN = FAN_FAST;
							}			 
							else if(cn0352.ch[chno].adp5065r.reg.ChargerSta2.BITs.THR_STATUS == THR_STATUS_BAT_OK){
								FAN = FAN_NORMAL;
							}
							break;
						}
						case FAN_NORMAL:{
							if((cn0352.ch[chno].adp5065r.reg.ChargerSta2.BITs.THR_STATUS == THR_STATUS_BAT_WARM) ||
								 (cn0352.ch[chno].adp5065r.reg.ChargerSta2.BITs.THR_STATUS == THR_STATUS_BAT_HOT)){
								FAN = FAN_FAST;
							}
							break;
						}
						case FAN_FAST:{
							break;
						}
						default:{
							break;
					}
				}
//			}
		}
	}
	switch(FAN){
		case FAN_SLOW: {		
			duty = 0.01;
			break;
		}
		case FAN_NORMAL:{
			//duty = 0.75;
			duty = 0.995;
			break;
		}
		case FAN_FAST:{
			duty = 0.995;
			break;
		}
		default:{
			duty = 0.75;
			break;
		}
	}
	pwm.updfreq(FAN_DEFAULT_FREQ,duty);
	return(duty);
};




