
/*
*===========================================================
*        File Name: i2c.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "i2c.h"
#include "hardware.h"


static int i2c_init(void);
static 	void i2c_x2id_wr(unsigned short id1, unsigned short id2, unsigned short data);
static 	void i2c_x1id_wr(unsigned short id, unsigned short data);
static 	unsigned char i2c_x2id_rd(unsigned short id1, unsigned short id2);
static 	unsigned char i2c_x1id_rd(unsigned short id);
static	I2CMSTA_T i2c_mstatus(void);
static unsigned char i2c_nackdata(unsigned short id);
static unsigned char i2c_nackaddr(unsigned short id);
extern void I2C0_Master_Int_Handler (void);

I2C_T i2c = {
	.init = i2c_init,
	.x2id_wr = i2c_x2id_wr,
	.x1id_wr = i2c_x1id_wr,
	.x2id_rd = i2c_x2id_rd,
	.x1id_rd = i2c_x1id_rd,
	.mstatus = i2c_mstatus,
	.nackdata = i2c_nackdata,
	.nackaddr = i2c_nackaddr,
	.Int_Handler = I2C0_Master_Int_Handler
};

static int i2c_init(void)
{
	//Configuration is only reasonable for I2C CLk is set to 16M
	I2cBaud(0x12, 0x13);	
	//Enable Transtation Complete Interrupt, Select Master Mode.
	I2cMCfg(0,I2CMCON_IENCMP_EN,I2CMCON_MAS_EN);
	return(0);
	
}
static 	void i2c_x2id_wr(unsigned short id1, unsigned short id2, unsigned short data)
{
	pADI_I2C->I2CMTX = id2;	//Prepare transmitted RegAddr
	pADI_I2C->I2CMTX = data;	//Prepare transmitted RegData	
	pADI_I2C->I2CADR0 = id1&0xFE;		
	while((pADI_I2C->I2CMSTA&I2CMSTA_BUSY)){}; //waiting till I2C master is not busy.
}
static 	void i2c_x1id_wr(unsigned short id, unsigned short data)
{
	pADI_I2C->I2CMTX = data;
	pADI_I2C->I2CADR0 = (id&0xFE);
	while((pADI_I2C->I2CMSTA&I2CMSTA_BUSY)){}; //waiting till I2C master is not busy.
}
static 	unsigned char i2c_x2id_rd(unsigned short id1, unsigned short id2)
{
	pADI_I2C->I2CMTX = id2;	//Prepare transmitted RegAddr
	pADI_I2C->I2CADR0 = id1&0xFE;
	while((pADI_I2C->I2CMSTA&I2CMSTA_BUSY)){}; //waiting till I2C master is not busy.	
	pADI_I2C->I2CADR0 = id1|0x01;
	while((pADI_I2C->I2CMSTA&I2CMSTA_BUSY)){}; //waiting till I2C master is not busy.	
	return(pADI_I2C->I2CMRX);
}
static 	unsigned char i2c_x1id_rd(unsigned short id)
{
	unsigned int i = 0;
	pADI_I2C->I2CADR0 = id|0x01;
	while((pADI_I2C->I2CMSTA&I2CMSTA_BUSY))
	{
		i++;
	}; //waiting till I2C master is not busy.	
	return(pADI_I2C->I2CMRX);
}
static	I2CMSTA_T i2c_mstatus(void){
	I2CMSTA_T temp;
	temp.REG.Val = pADI_I2C->I2CMSTA;
	return(temp);
};
static unsigned char i2c_nackdata(unsigned short id){
		i2c.wait_int = 1;
		NVIC_EnableIRQ(I2CM_IRQn);
		pADI_I2C->I2CADR0 = id;
		while(i2c.wait_int){};
		if(!i2c.I2CMSTA.REG.BITs.NACKDATA)
			return(1);
		else
			return(0);
};
static unsigned char i2c_nackaddr(unsigned short id){
		i2c.wait_int = 1;
		NVIC_EnableIRQ(I2CM_IRQn);
		pADI_I2C->I2CADR0 = id;
		while(i2c.wait_int){};
		if(!i2c.I2CMSTA.REG.BITs.NACKADDR)
			return(1);
		else
			return(0);
};
extern void I2C0_Master_Int_Handler (void)		//Master Translation Complete Inetrrupt.
{
	NVIC_DisableIRQ(I2CM_IRQn);
	i2c.I2CMSTA.REG.Val = pADI_I2C->I2CMSTA;
	i2c.wait_int = 0;
}
