/*
*===========================================================
*        File Name: pwm.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pwm.h"
#include "hardware.h"


static void pwm_init(void);
static void pwm_enable(void);
static void pwm_disable(void);
static void pwm_freqcfg(unsigned short highcnt, unsigned short count);
static void pwm_updfreq(float freq, float duty);
	
	
PWM_T pwm = {
	.init = pwm_init,
	.enable = pwm_enable,
	.disable = pwm_disable,
	.freqcfg = pwm_freqcfg,
	.updfreq = pwm_updfreq
};

static void pwm_init(void){
	pwm_disable();
	pADI_PWM->PWMCON0 = 0x0020;			//PWM CLK = 8MHz
	pADI_PWM->PWMCON1 = 0x0000;
	pADI_PWM->PWMCLRI = 0x0000;
	pwm_updfreq(25000,0.1);
};
static void pwm_enable(void){
	pADI_PWM->PWMCON0 |= 0x0001;
};
static void pwm_disable(void){
	pADI_PWM->PWMCON0 &= (~0x0001);
};
static void pwm_freqcfg(unsigned short highcnt, unsigned short count){
	pADI_PWM->PWM0LEN  = (unsigned short)count;
	pADI_PWM->PWM0COM0 = (unsigned short)count;
	pADI_PWM->PWM0COM1 = highcnt;
	pADI_PWM->PWM0COM2 = highcnt; // Rules: PWM0LEN >= PWM0COM0 >= PWM0COM1 >= PWM0COM2 >= 0x0000		
};
static void pwm_updfreq(float freq, float duty){
	unsigned short count = 0;
	unsigned short highcnt = 0;
	if(freq >0){
		count = (unsigned short)(8000000/freq);
		highcnt = (unsigned short)(count*duty);
		if((count != pwm.count)||(highcnt != pwm.highcnt)){
			pwm_disable();
			pwm_freqcfg(highcnt,count);
			pwm_enable();
			pwm.count = count;
			pwm.highcnt = highcnt;
		}
	}
};


