
/*
*===========================================================
*        File Name: timer0.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "timer0.h"
#include "hardware.h"
#include "task.h"



static void timer0_timeout(unsigned int timeoutfreq);
static void timer0_init(int timeout);
static void timer0_enable(void);
static void timer0_disable(void);
static void timer0_clear_int(void);
static void timer0_recount(void);
extern void GP_Tmr0_Int_Handler(void);

TIMER0_T timer0 = {
	.cnt = 0,
	.init = timer0_init,
	.timeout = timer0_timeout,	
	.enable = timer0_enable,
	.disable = timer0_disable,
	.clear_int = timer0_clear_int,
	.recount = timer0_recount,
	.Int_Handler = GP_Tmr0_Int_Handler
};

static void timer0_timeout(unsigned int timeoutfreq){
	TCON_T temp_tcon;
	int temp_timerout = 0;
	temp_tcon.Val = pADI_TM0->CON;
	switch(temp_tcon.BITs.PRE){
		case 0x00 : {
			temp_timerout = ADUCM360_PCLK;
			break;
		}
		case 0x01 : {
			temp_timerout = ADUCM360_PCLK/16;
			break;
		}
		case 0x02 : {
			temp_timerout = ADUCM360_PCLK/256;
			break;
		}
		case 0x03 : {
			temp_timerout = ADUCM360_PCLK/32768;
			break;
		}
		default : {
			temp_timerout = 0;
			break;
		}
	}
	temp_timerout /= timeoutfreq;
	pADI_TM0->LD = temp_timerout;
};
static void timer0_init(int timeout){
	timer0.cnt = 0;
	GptCfg(pADI_TM0,TCON_CLK_PCLK,TCON_PRE_DIV16,TCON_MOD_PERIODIC|TCON_RLD|TCON_EVENTEN_DIS|TCON_EVENTEN_DIS);	//Timer0 CLK = 16M/16 = 1Mhz
	GptLd(pADI_TM0,timeout); //Timer Out Frequency 5000Hz
}
static void timer0_enable(void){
	pADI_TM0->CON |= (unsigned short)TCON_ENABLE_MSK ;// Enable Timer
}
static void timer0_disable(void){
	pADI_TM0->CON &= (unsigned short)(~TCON_ENABLE_MSK) ;// Disable Timer
}
static void timer0_clear_int(void){
	pADI_TM0->CLRI = TSTA_TMOUT;	//Clear TimerOut Interrupt
}
static void timer0_recount(void){
	pADI_TM0->CON &= (unsigned short)(~TCON_ENABLE_MSK) ;// Disable Timer
	pADI_TM0->CLRI = TSTA_TMOUT;	//Clear TimerOut Interrupt
	pADI_TM0->CON |= (unsigned short)TCON_ENABLE_MSK ;// Enable Timer
}
//extern void GP_Tmr0_Int_Handler(void){
//}

